#!/usr/bin/env python
# encoding: utf-8
'''
metapylib.CovidCert -- shortdesc

metapylib.CovidCert is a description

It defines classes_and_methods

@author:     user_name

@copyright:  2021 organization_name. All rights reserved.

@license:    license

@contact:    user_email
@deffield    updated: Updated
'''

import sys
import os

import zlib
import pprint
import base45
import cbor2
import json
from typing import Union
from optparse import OptionParser

__all__ = []
__version__ = 0.1
__date__ = '2021-07-21'
__updated__ = '2021-07-21'

def b45decode(s: Union[bytes, str]) -> bytes:
    """Decode base45-encoded string to bytes"""
    BASE45_CHARSET = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ $%*+-./:"
    res = []
    try:
        if isinstance(s, str):
            buf = [BASE45_CHARSET.index(c) for c in s]
        else:
            buf = [BASE45_CHARSET.index(c) for c in s.decode()]
        buflen = len(buf)
        for i in range(0, buflen, 3):
            x = buf[i] + buf[i + 1] * 45
            if buflen - i >= 3:
                x = buf[i] + buf[i + 1] * 45 + buf[i + 2] * 45 * 45
                res.extend(list(divmod(x, 256)))
            else:
                res.append(x)
        return bytes(res)
    except (ValueError, IndexError, AttributeError):
        raise ValueError(".....................Invalid base45 string......")

def main(argv=None):
    '''Command line options.'''

    program_name = os.path.basename(sys.argv[0])
    program_version = "v0.1"
    program_build_date = "%s" % __updated__

    program_version_string = '%%prog %s (%s)' % (program_version, program_build_date)
    #program_usage = '''usage: spam two eggs''' # optional - will be autogenerated by optparse
    program_longdesc = '''''' # optional - give further explanation about what the program does
    program_license = "Copyright 2021"

    oResult = {"status":"fail","certType":"unknown","msg":"","data":{}}
    
    if argv is None:
        argv = sys.argv[1:]
    try:
        # setup option parser
        parser = OptionParser(version=program_version_string, epilog=program_longdesc, description=program_license)
        parser.add_option("-s", "--string", dest="strData", help="Covid-Cert QR-String", metavar="STRING")
        parser.add_option("-f", "--file", dest="filePath", help="Covid-Cert QR-File", metavar="FILE")
        parser.add_option("-p", "--pretty", action="store_true", dest="pPrint", help="Pretty Print", default=False)
    
    
        # process options
        (opts, args) = parser.parse_args(argv)
    
        #print(str(opts))
        strB45 = "";
        strInputData = "";
        if opts.strData:
            #test = '6BF+70790T9WJWG.FKY*4GO0.O1CV2 O5 N2FBBRW1*70HS8WY04AC*WIFN0AHCD8KD97TK0F90KECTHGWJC0FDC:5AIA%G7X+AQB9746HS80:54IBQF60R6$A80X6S1BTYACG6M+9XG8KIAWNA91AY%67092L4WJCT3EHS8XJC$+DXJCCWENF6OF63W5NW6WF6%JC QE/IAYJC5LEW34U3ET7DXC9 QE-ED8%E.JCBECB1A-:8$96646AL60A60S6Q$D.UDRYA 96NF6L/5QW6307KQEPD09WEQDD+Q6TW6FA7C466KCN9E%961A6DL6FA7D46JPCT3E5JDLA7$Q6E464W5TG6..DX%DZJC6/DTZ9 QE5$CB$DA/D JC1/D3Z8WED1ECW.CCWE.Y92OAGY8MY9L+9MPCG/D5 C5IA5N9$PC5$CUZCY$5Y$527B+A4KZNQG5TKOWWD9FL%I8U$F7O2IBM85CWOC%LEZU4R/BXHDAHN 11$CA5MRI:AONFN7091K9FKIGIY%VWSSSU9%01FO2*FTPQ3C3F'
            strInputData = opts.strData.strip()
            
        elif opts.filePath:
            with open(str(opts.filePath).strip(), 'r') as file:
                strInputData = file.read().replace('\n', '')
            
      
        
        if strInputData[3] == ":":
            splitInput = strInputData.split(":", 1);
            strB45 = splitInput[1]
            oResult["certType"] = splitInput[0]
        else:
            strB45 = strInputData
            
        #print("-------->"+strB45)
        zlibdata = base45.b45decode(strB45.strip())
        cbordata = zlib.decompress(zlibdata)
    
        decoded = cbor2.loads(cbordata)        
        # MAIN BODY #
        oResult["data"] = cbor2.loads(decoded.value[2])
        oResult["status"] = "success"
        #pprint.pprint(oResult)
        if opts.pPrint:
            print(json.dumps(oResult, indent=4, sort_keys=True))
        else:
            print(json.dumps(oResult))
            
        return 1

    except Exception as e:
        oResult["status"] = "fail"
        oResult["msg"] = str(e)
        #pprint.pprint(oResult)
        if opts.pPrint:
            print(json.dumps(oResult, indent=4, sort_keys=True))
        else:
            print(json.dumps(oResult))
        return 2
    


if __name__ == "__main__":
  
    sys.exit(main())