#!/bin/bash

set -e

dirname="$(dirname $0)"
cfgFile=$dirname/setup.cfg
nginxConfigDir=$dirname/ngnix

while getopts u:p:d:s: option
do
 case "${option}"
 in
 u) db_root_name=${OPTARG};;
 p) db_root_password=${OPTARG};;
 d) domain=${OPTARG};;
 s) skip=$OPTARG;;
 esac
done

if [ -f $cfgFile ]; then
    source $dirname/setup.cfg
fi

echo "                                          "
echo "┌┬┐┌─┐┌┬┐┌─┐╔═╗┌─┐┌─┐  ┌─┐┌─┐┌┬┐┬ ┬┌─┐";
echo "│││├┤  │ ├─┤╚═╗├┤ │    └─┐├┤  │ │ │├─┘";
echo "┴ ┴└─┘ ┴ ┴ ┴╚═╝└─┘└─┘  └─┘└─┘ ┴ └─┘┴  ";

if [ -z $skip ];then

    echo "Welcome to the metaSec james setup. This setup will help you to install James!"
    echo "Attention! This setup will drop (if excists) and create the james database. Type 'yes' to continue"

    read cont

    if ! [ "$cont" == "yes" ]; then
        echo "setup aborted"
        exit;
    fi
fi

until mysql -u $db_root_name  -e ";" ; do
        read -p "Please enter mysql user: " db_root_name
        read -s -p "Please enter mysql password: " db_root_password
done

echo " "

if [ -z $domain ];then
    echo -n "Please enter your james full domainname ( testname.example.com ): "
    read domain
fi

if [ -f "/opt/metasec_james/$domain" ]; then
    echo "Domain already excists"
        exit 1;
fi

db_password=$(cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w  16 | head -n 1)
echo "${dp_password}" > /etc/metasec/james_db_admin
db_user=$(echo "$domain" | cut -d. -f1)
db_name=$(echo "$db_user" | tr '-' '_')

james_user_name="james-adm@$domain"
james_user_password=$(cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w 16 | head -n 1)
james_user_password_db=$(echo -n $james_user_password | md5sum | awk '{print $1}')
#james_user_password="098f6bcd4621d373cade4e832627b4f6"

# echo "Drop database user $db_user ..."
# mysql -u $db_root_name  -e "DROP USER IF EXISTS '$db_user'@'localhost';"
echo "Create database user $db_user ..."
mysql -u $db_root_name  -e "CREATE USER IF NOT EXISTS '$db_user'@'localhost' IDENTIFIED BY '$db_password';"
# echo "Drop database $db_name ..."
# mysql -u $db_root_name  -e "DROP DATABASE IF EXISTS $db_name"
echo "Create database $db_name ..."
mysql -u $db_root_name  -e "CREATE DATABASE IF NOT EXISTS $db_name"
echo "Grant permissions ..."
mysql -u $db_root_name  -e "GRANT ALL ON $db_name.* to '$db_user'@'localhost';"

echo "create folder structure ..."
mkdir -p /opt/metasec_james/$domain

j_dirs="docs metadata templates templates/badges templates/pages templates/smarty  media wf_rules customportals contact_data"

for i in $j_dirs;do
    mkdir -p /opt/metasec_james/$domain/$i
    chown www-data:www-data /opt/metasec_james/$domain/$i
done

echo "create config file ..."
touch /opt/metasec_james/$domain/config.inc.php

echo "write config file ..."
echo "<?php" 													> /opt/metasec_james/$domain/config.inc.php
echo "\$CCBOX[\"SQL\"][\"DB\"]   = \"$db_name\";"				  >> /opt/metasec_james/$domain/config.inc.php
echo "\$CCBOX[\"SQL\"][\"USER\"] = \"$db_user\";" 				>> /opt/metasec_james/$domain/config.inc.php
echo "\$CCBOX[\"SQL\"][\"PASS\"] = \"$db_password\";"			>> /opt/metasec_james/$domain/config.inc.php

echo "Check shema for $domain ..."
cd /var/www/metasec_james_main/templates/db/
php check_shema.php $domain

echo "Create db james user $james_user_name ..."
mysql -u$db_user -p$db_password $db_name -e "INSERT IGNORE INTO persons
            (pe_id,pe_login_ok, pe_name, pe_vorname, pe_email, pe_passwd, pe_tz, pe_acl, pe_view_mode, pe_deleted, pe_individual_rights) VALUES
            (1,1,'James','Admin','$james_user_name','$james_user_password_db','Europe/Berlin','A,AC,ACL,GD','G',0,1);"
echo "Setting admin permissions for db james user $james_user_name ..."
mysql -u$db_user -p$db_password $db_name -e "REPLACE INTO person_access_level(pal_pe_id, pal_level)VALUES('1','A'),('1','AC'),('1','ACL'),('1','GD'),('1','HR');"
mysql -u$db_user -p$db_password $db_name -e "INSERT IGNORE INTO tenants (ten_id, ten_name, ten_descr, ten_url, ten_alias, ten_force_ssl, ten_status, ten_login_tpl, ten_on_invite_wf_id, ten_on_ckin_wf_id, ten_on_chkout_wf_id, ten_uuid) VALUES (1, '$domain', '$domain', '$domain', '$domain', 0, 'active', 0, 0, 0, 0, NULL);"
echo "Is this the default instance? (y/n)"

read cont

if [ "$cont" == "y" ]; then
    unlink /opt/metasec_james/$(ifconfig | grep -A2 eth| grep inet|head -n1| awk '{print $2}')
    ln -s /opt/metasec_james/$domain /opt/metasec_james/$(ifconfig | grep -A2 eth| grep inet|head -n1| awk '{print $2}')
fi

echo "Store login data into file? (y/n)"

read cont

if [ "$cont" == "y" ]; then
    echo "E-Mail: james-adm@$domain" >> /root/$domain
    echo "Password: $james_user_password" >> /root/$domain

    echo "login data stored at /root/$domain"
fi

echo "+++++++++++++++++"
echo "Login:"
echo "E-Mail: james-adm@$domain"
echo "Password: $james_user_password"
echo "+++++++++++++++++"

echo "Adjusting permissions"
chown -R www-data:www-data /opt/metasec_james/$domain


echo "Setup finished. Please restart the nginx service (service nginx restart)"

exit 0
