const name = "metaname";

export default {
    name,
    init() {
        // this is the reference to the svgEditor
        const svgEditor = this;

        // Deconstruct reference to the canvas
        const { svgCanvas } = svgEditor;

        // deconstruct the $id-function of the canvas to be able to select elements by their html-id
        const { $id } = svgCanvas;

        // Event-Handler that updates the selected element when the field is changed
        const nameChange = function() {
            const val = $id("meta_element_name").value ?? "";
            svgCanvas.changeSelectedAttribute("name", val);
            return true;
        };

        // function to draw the name field into the top bar (gets called when an element is being selected)
        const updateNameField = function() {

            // if the field exists, remove it
            if($id("meta_element_name_container"))
            {
                $id("meta_element_name_container").remove();
            }

            // draw the field only if an element is actually selected
            if(svgEditor.selectedElement)
            {
                // template for the input field
                const options = JSON.parse(document.body.dataset.nameopts);
                const takenNames = Object.values(document.querySelectorAll("svg *[name]")).map(elem => elem.getAttribute("name"));
                for(let name of takenNames)
                {
                    if(name!==svgEditor.selectedElement.getAttribute("name"))
                    {
                        delete options[name];
                    }
                }
                if(Object.keys(options).length<=1) return;

                const nameInput = `
                    <div class="selected_panel" id="meta_element_name_container">
                        <se-select id="meta_element_name" label="name" title="Name of the Element (makes the element selectable in echart)"
                            options="${Object.values(options).join(",")}"
                            values="${Object.keys(options).join("::")}"
                        >
                        </se-select>
                        <!--<se-input id="meta_element_name" size="10" label="name" title="Name of the Element (makes the element selectable in echart)"></se-input>-->
                    </div>
                `;

                // Add the input field to the top bar
                svgCanvas.insertChildAtIndex($id('tools_top'), nameInput, 200);

                // set the content of the input field to the name attribute of the selected element
                $id("meta_element_name").value = svgEditor.selectedElement.getAttribute("name") ?? "";

                // add the change-event-handler to the input field to update the svg-element when the value is changed
                $id("meta_element_name").addEventListener("change", nameChange.bind(this));
            }
        };

        return {
            name: name,
            // The callback should be used to load the DOM with the appropriate UI items
            callback() {
                updateNameField.call(this);
            },

            // hook the field-render function to the "selectedChanged"-Event of the editor
            selectedChanged() {
                updateNameField.call(this);
            }
        };
    }
};
