/**
 * @file ext-metasave.js
 *
 * @license MIT
 *
 * @copyright 2021 metaSEC e.K.
 *
 */

 const name = "metasave";

 const loadExtensionTranslation = async function (svgEditor) {
   let translationModule;
   const lang = svgEditor.configObj.pref('lang');
   try {
     // eslint-disable-next-line no-unsanitized/method
     translationModule = await import(`./locale/${lang}.js`);
   } catch (_error) {
     // eslint-disable-next-line no-console
     console.warn(`Missing translation (${lang}) for ${name} - using 'en'`);
     // eslint-disable-next-line no-unsanitized/method
     translationModule = await import(`./locale/en.js`);
   }
   svgEditor.i18next.addResourceBundle(lang, 'translation', translationModule.default, true, true);
 };

 export default {
   name,
   async init(_S) {
     const svgEditor = this;
     const { svgCanvas } = svgEditor;
     const { $id } = svgCanvas;
     await loadExtensionTranslation(svgEditor);

     const clickDocProps = function() {
         $id("tool_docprops").click();
     }

     /**
      *
      * @returns {void}
      */
     const clickSave = async function () {
       const $editorDialog = $id("se-svg-editor-dialog");
       const editingsource = $editorDialog.getAttribute("dialog") === "open";
       if (editingsource) {
         svgEditor.saveSourceEditor();
       } else {
         // In the future, more options can be provided here
         const saveOpts = {
           images: svgEditor.configObj.pref("img_save"),
           round_digits: 6
         };
         // remove the selected outline before serializing
         svgCanvas.clearSelection();
         // Update save options if provided
         if (saveOpts) {
           const saveOptions = svgCanvas.mergeDeep(svgCanvas.getSvgOption(), saveOpts);
           for (const [ key, value ] of Object.entries(saveOptions)) {
             svgCanvas.setSvgOption(key, value);
           }
         }
         svgCanvas.setSvgOption('apply', true);

         // no need for doctype, see https://jwatt.org/svg/authoring/#doctype-declaration
         const svg = '<?xml version="1.0"?>\n' + svgCanvas.svgCanvasToString();

         fetch(`/portals/save_svg.php${window.location.search}`, {
             method:"POST",
             headers:{
                 'Content-Type': 'application/json'
             },
             body: JSON.stringify({
                 svgcontent: svg
             })
         }).then(async (res) => {
             const data = await res.json();
             console.log("response", res, data);
             alert("SVG Saved.");
             return res;
         }).catch((err) => {
             console.error("error", err);
             alert("An error occured. Information can be found in the JavaScript Console.");
         });
       }
     };

     return {
       name: 'metasave',
       // The callback should be used to load the DOM with the appropriate UI items
       callback() {
         // eslint-disable-next-line no-unsanitized/property
         const saveButtonTemplate = `<se-menu-item id="tool_save" label="opensave.save_doc" shortcut="S" src="saveImg.svg"></se-menu-item>`;
         svgCanvas.insertChildAtIndex($id('main_button'), saveButtonTemplate, 2);

         const docPropsButtonTemplate = `<se-button id="tool_docprops_open" title="tools.docprops" src="docprop.svg"></se-button>`;
         svgCanvas.insertChildAtIndex($id("editor_panel"),docPropsButtonTemplate,0);

         const fastSaveButtonTemplate = `<se-button id="tool_fastsave" title="opensave.save_doc" shortcut="S" src="floppy.svg"></se-button>`;
         svgCanvas.insertChildAtIndex($id("editor_panel"),fastSaveButtonTemplate,0);


         // handler
         $id("tool_save").addEventListener("click", clickSave.bind(this));
         $id("tool_fastsave").addEventListener("click", clickSave.bind(this));
         $id("tool_docprops_open").addEventListener("click", clickDocProps.bind(this));
       }
     };
   }
 };
